import { Store } from 'redux';
import { NormalAction, IActionNormal } from '../actions/NormalAction';
import { ComposeAction } from '../actions/ComposeAction';
import { HttpServiceBuilderWithMeta } from '../services/HttpServiceBuilder';
import { IReducers } from '../reducers/BaseReducer';
export declare type FilterPersist<Data> = (state: State<Data>) => StateReturn<Data>;
export declare type AnyModel = BaseModel<any>;
export declare type State<Data> = Data & {
    /**
     * `Modify data directly like this:`
     * ```javascript
     * this.action((state, i: number) => {
     *   state.amount += i;
     *   state.foo.bar = { is: 'cool' };
     *   state.names.push('dc');
     *   // Don't return state above.
     *   // But new state must return.
     * });
     * ```
     */
    readonly __mvvm: 'Modify data directly';
};
export declare type StateReturn<Data> = void | (Data & {
    readonly __mvvm?: 'Don\'t return state unless it\'s new';
});
export declare type Effects<Data> = Array<{
    when: string;
    effect: (state: State<Data>, action: any) => StateReturn<Data>;
}>;
export declare type CreateNormalActionPayload<A> = A extends (state: any, payload: infer P) => any ? P : never;
export declare type CreateNormalActionEffect<Data, A> = A extends (state: any, ...args: infer P) => any ? (...args: P) => IActionNormal<Data, P[0]> : never;
export declare abstract class BaseModel<Data = null, RequestOption extends object = object> {
    private readonly __name;
    private readonly __alias;
    private __initData?;
    private __anonymousAction?;
    /**
     * Filter data from storage. Assign model to allowlist before you can use persist:
     * ```javascript
     * const store = createReduxStore({
     *   persist: {
     *     allowlist: {
     *       xxxModel,
     *       yyyModel,
     *     }
     *   }
     * });
     * ```
     *
     * MVVM is allowed here.
     */
    protected filterPersistData?: FilterPersist<Data>;
    constructor(alias?: string);
    getReducerName(): string;
    get data(): Data extends null ? never : Data;
    reset(): void;
    protected changeReducer(fn: (state: State<Data>) => StateReturn<Data>): void;
    protected action<Fn extends (state: State<Data>, payload: any) => StateReturn<Data>>(effect: Fn): CreateNormalActionEffect<Data, Fn> & NormalAction<Data, Fn, CreateNormalActionPayload<Fn>>;
    protected compose<Fn extends (...args: any[]) => Promise<any>>(fn: Fn): Fn & ComposeAction<Data, Fn>;
    protected effects(): Effects<Data>;
    protected get<Response>(uri: string): HttpServiceBuilderWithMeta<Data, Response, unknown, RequestOption>;
    protected post<Response>(uri: string): HttpServiceBuilderWithMeta<Data, Response, unknown, RequestOption>;
    protected put<Response>(uri: string): HttpServiceBuilderWithMeta<Data, Response, unknown, RequestOption>;
    protected delete<Response>(uri: string): HttpServiceBuilderWithMeta<Data, Response, unknown, RequestOption>;
    protected patch<Response>(uri: string): HttpServiceBuilderWithMeta<Data, Response, unknown, RequestOption>;
    protected connect<Response>(uri: string): HttpServiceBuilderWithMeta<Data, Response, unknown, RequestOption>;
    register(): IReducers;
    protected autoRegister(): boolean;
    protected onStoreCreated(store: Store): void;
    protected abstract initReducer(): Data;
    private __createHttpServiceBuilder;
    private __getInitData;
}
