"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BaseReducer = void 0;
var immer_1 = require("immer");
var StoreHelper_1 = require("../stores/StoreHelper");
var StateReturnRequiredError_1 = require("../exceptions/StateReturnRequiredError");
var Persist_1 = require("../stores/Persist");
var BaseReducer = /** @class */ (function () {
    function BaseReducer(model, initData) {
        this.sideEffects = {};
        this.model = model;
        this.initData = initData;
        this.reducerName = model.getReducerName();
        this.sideEffects = model.effects().reduce(function (carry, _a) {
            var when = _a.when, effect = _a.effect;
            carry[when] = effect;
            return carry;
        }, {});
    }
    BaseReducer.prototype.createReducer = function () {
        var _a;
        if (this.initData === null) {
            return {};
        }
        return _a = {},
            _a[this.reducerName] = this.reducer.bind(this),
            _a;
    };
    BaseReducer.prototype.reducer = function (state, action) {
        if (state === undefined) {
            var newState = StoreHelper_1.storeHelper.persist.getPersistData(this.reducerName, this.initData);
            return this.initFromPersist(newState);
        }
        // For async storage, we should dispatch action to inject persist data into reducer
        if (action.type === Persist_1.TYPE_REHYDRATE && action.payload && action.payload[this.reducerName] !== undefined) {
            return this.initFromPersist(action.payload[this.reducerName]);
        }
        if (action.modelName === this.reducerName) {
            if (action.effect) {
                return this.changeState(action.effect, state, action);
            }
        }
        else if (this.sideEffects[action.type]) {
            return this.changeState(this.sideEffects[action.type], state, action);
        }
        return state;
    };
    BaseReducer.prototype.initFromPersist = function (state) {
        if (this.initData === state || !this.model.filterPersistData) {
            return state;
        }
        return this.changeState(this.model.filterPersistData, state, {
            type: 'filterPersistData',
            modelName: this.reducerName,
            payload: undefined,
            effect: this.model.filterPersistData,
        });
    };
    ;
    BaseReducer.prototype.changeState = function (effect, state, action) {
        if (immer_1.isDraftable(state)) {
            var draft = immer_1.createDraft(state);
            var responseDraft = effect(draft, action);
            if (responseDraft === undefined) {
                state = immer_1.finishDraft(draft);
            }
            else if (immer_1.isDraft(responseDraft)) {
                state = immer_1.finishDraft(responseDraft);
            }
            else {
                state = responseDraft;
            }
        }
        else {
            state = effect(state, action);
            if (state === undefined) {
                throw new StateReturnRequiredError_1.StateReturnRequiredError(action.type);
            }
        }
        return state;
    };
    return BaseReducer;
}());
exports.BaseReducer = BaseReducer;
