"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.HttpServiceBuilder = void 0;
var HttpServiceBuilder = /** @class */ (function () {
    function HttpServiceBuilder(config) {
        this.config = config;
    }
    HttpServiceBuilder.prototype.query = function (query) {
        this.config.query = query;
        return this;
    };
    HttpServiceBuilder.prototype.body = function (body) {
        this.config.body = body;
        return this;
    };
    HttpServiceBuilder.prototype.successText = function (text) {
        this.config.successText = text;
        return this;
    };
    HttpServiceBuilder.prototype.failText = function (text) {
        this.config.failText = text;
        return this;
    };
    HttpServiceBuilder.prototype.requestOptions = function (options) {
        this.config.requestOptions = options;
        return this;
    };
    HttpServiceBuilder.prototype.hideError = function (is) {
        this.config.hideError = is;
        return this;
    };
    HttpServiceBuilder.prototype.throttle = function (options) {
        var obj = typeof options === 'number' ? { duration: options } : options;
        this.config.useThrottle = obj.enable !== false;
        this.config.throttleMillSeconds = obj.duration;
        this.config.throttleDeps = obj.deps;
        return this;
    };
    HttpServiceBuilder.prototype.payload = function (payload) {
        // @ts-ignore
        // @ts-expect-error
        this.config.payload = payload;
        // @ts-ignore
        // @ts-expect-error
        return this;
    };
    HttpServiceBuilder.prototype.metas = function (value) {
        this.config.metaKey = value;
        // @ts-ignore
        // @ts-expect-error
        return this;
    };
    ;
    HttpServiceBuilder.prototype.onPrepare = function (fn) {
        this.config.onPrepare = fn;
        return this;
    };
    HttpServiceBuilder.prototype.onSuccess = function (fn) {
        this.config.onSuccess = fn;
        return this;
    };
    HttpServiceBuilder.prototype.onFail = function (fn) {
        this.config.onFail = fn;
        return this;
    };
    HttpServiceBuilder.prototype.collect = function (instance) {
        var config = this.config;
        var action = {
            uniqueId: instance.uniqueId,
            uri: config.uri,
            type: {
                prepare: instance.getPrepareType(),
                success: instance.getSuccessType(),
                fail: instance.getFailType(),
            },
            method: config.method,
            modelName: config.instanceName,
            payload: config.payload,
            body: config.body || {},
            query: config.query || {},
            successText: config.successText || '',
            failText: config.failText || '',
            hideError: config.hideError || false,
            requestOptions: config.requestOptions || {},
            metaKey: config.metaKey === undefined ? true : config.metaKey,
            metaActionName: instance.getActionName(),
            onPrepare: config.onPrepare || null,
            onSuccess: config.onSuccess || null,
            onFail: config.onFail || null,
            useThrottle: config.useThrottle || false,
            throttleMillSeconds: config.throttleMillSeconds || 0,
            throttleKey: '',
            throttleDeps: config.throttleDeps || [],
        };
        return action;
    };
    return HttpServiceBuilder;
}());
exports.HttpServiceBuilder = HttpServiceBuilder;
