"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DynamicMiddleware = void 0;
var tslib_1 = require("tslib");
var redux_1 = require("redux");
var DynamicMiddleware = /** @class */ (function () {
    function DynamicMiddleware() {
        this.items = {};
    }
    DynamicMiddleware.prototype.create = function () {
        var _this = this;
        this.resetCache();
        return function (api) { return function (next) { return function (action) {
            return _this.getDispatch(api, next)(action);
        }; }; };
    };
    DynamicMiddleware.prototype.use = function (id, middleware) {
        this.resetCache();
        this.items[id] = middleware;
        return this;
    };
    DynamicMiddleware.prototype.resetCache = function () {
        this.dispatch = undefined;
    };
    DynamicMiddleware.prototype.getDispatch = function (api, next) {
        var _this = this;
        if (!this.dispatch) {
            var chain = Object.keys(this.items).map(function (key) {
                return _this.items[key](api);
            });
            this.dispatch = redux_1.compose.apply(void 0, tslib_1.__spread(chain))(next);
        }
        return this.dispatch;
    };
    return DynamicMiddleware;
}());
exports.DynamicMiddleware = DynamicMiddleware;
